; *********************************************************************************
;
; Ansicht von unten: DS18x20
;
;     GND   DQ    Vdd
;       -----------
;       | 1  2  3 |
;       \         /
;        \_______/
;
; *********************************************************************************
; OneWire Port Register
.equ    OW_DDR      = DDRB
.equ    OW_IN       = PINB
.equ    OW_PIN      = PB0
; -------------------------------------------
.equ    CRC_POLYNOM = 0b10001100
; *********************************************************************************

.dseg
; ID Datenpaket
ID_Family_Code:     .byte 1                 ; Family_Code
ID_Serial:          .byte 6                 ; Serial Number 1-6
ID_CRC:             .byte 1                 ; CRC

; Scratchpad Datenpaket
SP_Temp_LSB:        .byte 1                 ; Temperature LSB
SP_Temp_MSB:        .byte 1                 ; Temperature MSB
SP_TH:              .byte 1                 ; TH/User Byte 1
SP_TL:              .byte 1                 ; TL/User Byte 2
SP_RES:             .byte 2                 ; reserviert
SP_Count_Remain:    .byte 1                 ; Count Remain 0 - 15 
SP_Count_per_C:     .byte 1                 ; Count per C, immer 0x10
SP_CRC:             .byte 1                 ; CRC

.cseg
; *********************************************************************************
DS18x20_read_id:
; *********************************************************************************
    rcall   onewire_reset
    ldi     Temp3, 0x33                     ; OneWire Kommando fr ROM lesen
    rcall   ONEWIRE_WR                      ; sende ein Kommando
    ldi     ZL, low(ID_Family_Code)         ; Zeiger auf Datenpaket
    ldi     ZH, high(ID_Family_Code)
    ldi     Temp1, 8                        ; Bytezhler
    rcall   onewire_daten_lesen
    cbr     Fehler, 1<<CRC_ID               ; lsche ID CRC Fehler
    cpi     CRC, 0                          ; ist CRC = 0
    breq    DS18x20_read_id_CRC_OK          ; dann CRC ist O.K.
    sbr     Fehler, 1<<CRC_ID               ; setze ID CRC Fehler
DS18x20_read_id_CRC_OK:
    ret

; *********************************************************************************
DS18x20_read_scratchpad:
; *********************************************************************************
    rcall   onewire_reset
    ldi     Temp3, 0xCC                     ; skip ROM Command
    rcall   ONEWIRE_WR
    ldi     Temp3, 0x44                     ; convert Temperature Command
    rcall   ONEWIRE_WR

    ldi     temp2, 250
    rcall   warte_Nx1ms
    ldi     temp2, 250
    rcall   warte_Nx1ms
    ldi     temp2, 250
    rcall   warte_Nx1ms                     ; max. Wandlungszeit 750ms!

    rcall   onewire_reset
    ldi     Temp3, 0xCC                     ; skip ROM Command
    rcall   ONEWIRE_WR
    ldi     Temp3, 0xBE                     ; read scratchpad Command
    rcall   ONEWIRE_WR
    ldi     ZL, low(SP_Temp_LSB)            ; Zeiger laden
    ldi     ZH, high(SP_Temp_LSB)           ; 
    ldi     Temp1, 9                        ; Bytezhler
    rcall   onewire_daten_lesen
    cbr     Fehler, 1<<CRC_SP               ; lsche SP CRC Fehler
    cpi     CRC, 0                          ; ist CRC = 0
    breq    DS18x20_read_scratchpad_CRC_OK  ; dann ist CRC O.K.
    sbr     Fehler, 1<<CRC_SP               ; setze SP CRC Fehler
DS18x20_read_scratchpad_CRC_OK:
    ret

; *********************************************************************************
onewire_daten_lesen:
; Parameter:  Z: Zeiger auf RAM, temp1: Bytezhler;
;----------------------------------------------------------------------------------
    clr     CRC                             ; CRC lschen!
onewire_daten_lesen_schleife:
    rcall   ONEWIRE_RD                      ; Byte vom OneWire Bus lesen
    st      Z+, Temp3                       ; speichern
    rcall   onewire_CRC_berechnen
    dec     Temp1                           ; Schleifenzhler
    brne    onewire_daten_lesen_schleife    ; Schleifenende
    ret

; *********************************************************************************
onewire_CRC_berechnen:                      ; Berechne CRC fr ein Byte
; Parameter: temp3: Daten
; ---------------------------------------------------------------------------------
    ldi     Temp0, 8                        ; Bitzhler
onewire_CRC_berechnen_schleife:
    lsr     Temp3                           ; LSB der Daten ins Carry Bit schieben
    ror     CRC                             ; Carry Bit in MSB der CRC schieben, LSB der CRC geht ins Carry Bit
    brcc    onewire_CRC_berechnen_kein_XOR  ; verzweigen, wenn Carry gelscht ist
    eor     CRC, Divisor                    ; exklusive ODER-Verknpfung mit CRC Polynom
onewire_CRC_berechnen_kein_XOR:
    dec     Temp0                           ; Schleifenzhler
    brne    onewire_CRC_berechnen_schleife  ; Schleifenende
    ret

; *********************************************************************************
onewire_reset:
; *********************************************************************************
;    
;          _                          ____         _______________    
; RESET     |                        |////|       |////////////|    
;           |________________________|////|_______|////////////|    
;            <--------480s----------><-64s->^--Wait Release-->
;                                             ^
;                                             ^
;                                           Sample
;----------------------------------------------------------------------------------
    sbi     OW_DDR, OW_PIN              ; OneWire Pin LOW
    rcall   warte_480us
    cbi     OW_DDR, OW_PIN              ; OneWire High-Z, externer Pull Up Widerstand zieht auf HIGH
    ldi     temp2, 7
    rcall   warte_Nx10us                ; Warte 70us
    cbr     Fehler, (1<<OW_RESET_LOW) | (1<<OW_RESET_HIGH)  ; lsche Fehler
    sbic    OW_IN, OW_PIN               ; berspringe nchsten Befehl, wenn DS1820 den Bus auf LOW zieht
    sbr     Fehler, 1<<OW_RESET_HIGH    ; wenn nein, dann setze Fehler, Bus ist HIGH, IC reagiert nicht
    rcall   warte_480us
    sbis    OW_IN, OW_PIN               ; berspringe nchsten Befehl, wenn DS1820 den Bus freigibt, Pull Up Widerstand zieht auf HIGH
    sbr     Fehler, 1<<OW_RESET_LOW     ; wenn nein, setze Fehler, Bus ist LOW (Kurzschlu nach GND?)
    ret

; *********************************************************************************
onewire_wr:                                 ; sende ein Byte auf OneWire Bus
;Parameter: temp3: Daten
; *********************************************************************************
;           _         _____________________________________       
; Write 1    |       |                                     |
; -------    |_______|                                     |
;            |<-4s->|<----------90s--------------------->|    
;
;------------------------------------------------------------------     
;           _                                    _______    
; Write 0    |                                  |       |
; -------    |__________________________________|       |
;            |<---------------74s------------->|<-20s>|     
;
;----------------------------------------------------------------------------------
    ldi     temp0, 8            ; Bitzhler
onewire_wr_schleife:
    sbi     OW_DDR, OW_PIN      ; OneWire auf LOW
    rcall   warte_4us
    ror     temp3               ; nchstes Bit ins carry Bit schieben
    brcc    onewire_wr_null     ; springe, wenn carry = 0
    cbi     OW_DDR, OW_PIN      ; OneWire auf High-Z -> Pull Up zieht auf HIGH
onewire_wr_null:
    ldi     temp2, 7
    rcall   warte_Nx10us        ; warte 70us
    cbi     OW_DDR, OW_PIN      ; OneWire auf High-Z -> Pull Up zieht auf HIGH
    ldi     temp2, 2
    rcall   warte_Nx10us        ; warte 20us
    dec     temp0               ; Schleifenzhler
    brne    onewire_wr_schleife ; Schleifenende
    ret

; *********************************************************************************
onewire_rd:                                 ; lies ein Bytes vom OneWire Bus
; Rckgabe: temp3: Daten
; *********************************************************************************
;           _         ____________________________________________      
; READ       |       |///////////////////////////////////////////|    
;            |______ |///////DS18B20 zieht auf Low/High//////////|    
;            |<-4s->|<-10s-->|<--------------80s------------->|    
;                              ^
;                              ^
;                           Sample
;------------------------------------------------------------------
    ldi     Temp0, 8            ; Bitzhler
onewire_rd_schleife:
    sbi     OW_DDR, OW_PIN      ; OneWire auf LOW
    rcall   warte_4us
    cbi     OW_DDR, OW_PIN      ; OneWire auf High-Z -> Pull Up zieht auf HIGH
    ldi     temp2, 1
    rcall   warte_Nx10us        ; warte 10us
    clc                         ; carry Bit = 0
    sbic    OW_IN, OW_PIN       ; OneWire Bus = LOW ?
    sec                         ; wenn nein, carry = 1
    ror     temp3               ; carry Bit als MSB in temp3 schieben
    ldi     temp2, 8
    rcall   warte_Nx10us        ; warte 80us
    dec     temp0               ; Schleifenzhler
    brne    onewire_rd_schleife ; Schleifenende
    ret

; *********************************************************************************

