;**************************************************************************
; http://mikrocontroller-blog.de/module/displays/hd44780-1602-lcd-modul-mit-i2c-schnittstelle/
;**************************************************************************
;
;                PCF8574
;               ---------
;              |*        |
;           A0 |1      16| VCC
;           A1 |2      15| SDA
;           A2 |3      14| SCL
;           P0 |4      13| INT
;           P1 |5      12| P7
;           P2 |6      11| P6
;           P3 |7      10| P5
;          GND |8       9| P4
;              |         |
;               ---------
;
;                           ----------------------------------------
; PCF8574 I2C Addressbyte:  | 0 | 1 | 0 | 0 | 0 | A2 | A1 | A0 |R/W|
;                           ----------------------------------------
.set    I2C_Adresse     = 0x4E          ; PCF8574, 8 Bit Adresse mit Bit#0 = 0!
;
; 100 kHz I2C-Bus-Schnittstelle
;*************************************************************************
/*
 Datenbyte 0xAA bertragen
   __      ______      ______      ______      ______                 ___
SDA  \____/      \____/      \____/      \____/      \_______________/
     Start    7     6     5     4     3     2     1     0    ACK   Stop
   ____      __    __    __    __    __    __    __    __    __     _____
SCL    \____/  \__/  \__/  \__/  \__/  \__/  \__/  \__/  \__/  \___/
*/
;**************************************************************************
I2C_Start_Adresse:              ; I2C Start + Adresse senden
;--------------------------------------------------------------------------
    sbi     PORT_DDR, SDA       ; SDA LOW, Startbedingung
    rcall   warte_4us
; ------------------------------
    sbi     Port_DDR, SCL       ; SCL LOW
    rcall   warte_4us
; ------------------------------
    push    Temp1
    ldi     Temp1, I2C_Adresse
    rcall   I2C_Write           ; 
    pop     Temp1
; ------------------------------
    ret

;*************************************************************************
I2C_Write:                      ; ein Byte auf I2C Bus schreiben
; Parameter: temp1: Daten (bleiben nach dem Funktionsaufruf erhalten!)
; Rckgabe: Carry-Bit: ACK-Bit vom Slave, 0 == ACK, 1 = NACK
;--------------------------------------------------------------------------
    push    temp2               ;
    ldi     temp2, 8            ; Bitzhler
; ------------------------------
I2C_Write_Loop:
    sbrc    temp1, 7            ; berspringen, wenn LOW
    cbi     Port_DDR, SDA       ; MSB ist HIGH, setzte SDA HIGH-Z, externer Pull Up Widerstand zieht auf HIGH
    sbrs    temp1, 7            ; berspringen, wenn HIGH
    sbi     Port_DDR, SDA       ; MSB ist LOW, setzte SDA LOW
; ------------------------------
I2C_Write_HIGH:
    rol     temp1               ; nchstes Bit ins MSB schieben
    rcall   warte_4us
    cbi     Port_DDR, SCL       ; SCL auf HIGH-Z, externer Pull Up Widerstand zieht auf HIGH
; ------------------------------
    rcall   warte_4us
    sbi     Port_DDR, SCL       ; SCl auf LOW
; ------------------------------
    rcall   warte_4us
    dec     Temp2
    brne    I2C_Write_Loop
; ------------------------------
    rol     temp1               ; 9 mal rotiert, temp1 ist wieder das Original
    pop     temp2

;--------------------------------------------------------------------------
Acknowledge:                    ; Besttigung vom I2C Slave lesen
;--------------------------------------------------------------------------
    cbi     Port_DDR, SDA       ; SDA auf HIGH-Z
    rcall   warte_4us
    cbi     Port_DDR, SCL       ; SCL auf HIGH-Z, externer Pull Up Widerstand zieht auf HIGH
    rcall   warte_4us
    sec                         ; Carry Bit aus 1 setzen
    sbis    PORT_IN, SDA        ; berspringen, wenn SDA = High
    clc                         ; Rckgabewert im Carry Bit, 0 == ACK, 1 = NACK
    sbi     Port_DDR, SCL       ; SCL auf LOW
    rcall   warte_4us
    cbi     PORT_DDR, SDA       ; SDA auf HIGH-Z
    rcall   warte_4us
    ret

;**************************************************************************
I2c_Stop:                       ; 
;--------------------------------------------------------------------------
    sbi     Port_DDR, SDA       ; SDA = LOW
    rcall   warte_4us
    cbi     Port_DDR, SCL       ; SCL = HIGH-Z, externer Pull Up Widerstand zieht auf HIGH
    rcall   warte_4us
    cbi     Port_DDR, SDA       ; SDA = HIGH-Z, externer Pull Up Widerstand zieht auf HIGH
    rcall   warte_4us
    ret

;*************************************************************************
